@@ -1,4 +1,4 @@
-/* $OpenBSD: dsa.c,v 1.9 2017/01/20 08:57:11 deraadt Exp $ */
+/* $OpenBSD: dsa.c,v 1.11 2018/02/07 05:47:55 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -360,12 +360,10 @@ dsa_main(int argc, char **argv)
 		ERR_print_errors(bio_err);
 	} else
 		ret = 0;
-end:
+ end:
 	BIO_free(in);
-	if (out != NULL)
-		BIO_free_all(out);
-	if (dsa != NULL)
-		DSA_free(dsa);
+	BIO_free_all(out);
+	DSA_free(dsa);
 	free(passin);
 	free(passout);
 
