@@ -1,4 +1,4 @@
-/* $OpenBSD: genrsa.c,v 1.9 2017/01/20 08:57:12 deraadt Exp $ */
+/* $OpenBSD: genrsa.c,v 1.11 2018/02/07 05:47:55 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -165,7 +165,7 @@ genrsa_main(int argc, char **argv)
 		argc--;
 	}
 	if ((argc >= 1) && ((sscanf(*argv, "%d", &num) == 0) || (num < 0))) {
-bad:
+ bad:
 		BIO_printf(bio_err, "usage: genrsa [args] [numbits]\n");
 		BIO_printf(bio_err, " -des            encrypt the generated key with DES in cbc mode\n");
 		BIO_printf(bio_err, " -des3           encrypt the generated key with DES in ede cbc mode (168 bit key)\n");
@@ -233,14 +233,12 @@ bad:
 	}
 
 	ret = 0;
-err:
-	if (bn)
-		BN_free(bn);
-	if (rsa)
-		RSA_free(rsa);
-	if (out)
-		BIO_free_all(out);
+ err:
+	BN_free(bn);
+	RSA_free(rsa);
+	BIO_free_all(out);
 	free(passout);
+
 	if (ret != 0)
 		ERR_print_errors(bio_err);
 
