@@ -1,4 +1,4 @@
-/* $OpenBSD: rand.c,v 1.11 2017/01/20 08:57:12 deraadt Exp $ */
+/* $OpenBSD: rand.c,v 1.13 2018/02/07 05:47:55 jsing Exp $ */
 /* ====================================================================
  * Copyright (c) 1998-2001 The OpenSSL Project.  All rights reserved.
  *
@@ -176,10 +176,9 @@ rand_main(int argc, char **argv)
 
 	ret = 0;
 
-err:
+ err:
 	ERR_print_errors(bio_err);
-	if (out)
-		BIO_free_all(out);
+	BIO_free_all(out);
 
 	return (ret);
 }
